@@ -256,15 +256,17 @@ class LinuxInterfaceDriver(object):
 
     @abc.abstractmethod
     def plug_new(self, network_id, port_id, device_name, mac_address,
-                 bridge=None, namespace=None, prefix=None, mtu=None):
+                 bridge=None, namespace=None, prefix=None, mtu=None,
+                 link_up=True):
         """Plug in the interface only for new devices that don't exist yet."""
 
     def plug(self, network_id, port_id, device_name, mac_address,
-             bridge=None, namespace=None, prefix=None, mtu=None):
+             bridge=None, namespace=None, prefix=None, mtu=None, link_up=True):
         if not ip_lib.device_exists(device_name,
                                     namespace=namespace):
-            self.plug_new(network_id, port_id, device_name, mac_address,
-                          bridge, namespace, prefix, mtu)
+            self._safe_plug_new(
+                network_id, port_id, device_name, mac_address, bridge,
+                namespace, prefix, mtu, link_up)
         else:
             LOG.info("Device %s already exists", device_name)
             if mtu:
@@ -273,6 +275,22 @@ class LinuxInterfaceDriver(object):
             else:
                 LOG.warning("No MTU configured for port %s", port_id)
 
+    def _safe_plug_new(self, network_id, port_id, device_name, mac_address,
+            bridge=None, namespace=None, prefix=None, mtu=None, link_up=True):
+        try:
+            self.plug_new(
+                network_id, port_id, device_name, mac_address, bridge,
+                namespace, prefix, mtu, link_up)
+        except TypeError:
+            LOG.warning("Interface driver's plug_new() method should now "
+                        "accept additional optional parameter 'link_up'. "
+                        "Usage of plug_new() method which takes from 5 to 9 "
+                        "positional arguments is now deprecated and will not "
+                        "be possible in W release.")
+            self.plug_new(
+                network_id, port_id, device_name, mac_address, bridge,
+                namespace, prefix, mtu)
+
     @abc.abstractmethod
     def unplug(self, device_name, bridge=None, namespace=None, prefix=None):
         """Unplug the interface."""
@@ -300,10 +318,21 @@ class LinuxInterfaceDriver(object):
             LOG.warning("Interface driver cannot update MTU for ports")
             self._mtu_update_warn_logged = True
 
+    def set_link_status(self, device_name, namespace=None, link_up=True):
+        ns_dev = ip_lib.IPWrapper(namespace=namespace).device(device_name)
+        if not ns_dev.exists():
+            LOG.debug("Device %s may concurrently be deleted.", device_name)
+            return
+        if link_up:
+            ns_dev.link.set_up()
+        else:
+            ns_dev.link.set_down()
+
 
 class NullDriver(LinuxInterfaceDriver):
     def plug_new(self, network_id, port_id, device_name, mac_address,
-                 bridge=None, namespace=None, prefix=None, mtu=None):
+                 bridge=None, namespace=None, prefix=None, mtu=None,
+                 link_up=True):
         pass
 
     def unplug(self, device_name, bridge=None, namespace=None, prefix=None):
@@ -338,7 +367,8 @@ class OVSInterfaceDriver(LinuxInterfaceDriver):
         ovs.replace_port(device_name, *attrs)
 
     def plug_new(self, network_id, port_id, device_name, mac_address,
-                 bridge=None, namespace=None, prefix=None, mtu=None):
+                 bridge=None, namespace=None, prefix=None, mtu=None,
+                 link_up=True):
         """Plug in the interface."""
         if not bridge:
             bridge = self.conf.ovs_integration_bridge
@@ -402,7 +432,8 @@ class OVSInterfaceDriver(LinuxInterfaceDriver):
         else:
             LOG.warning("No MTU configured for port %s", port_id)
 
-        ns_dev.link.set_up()
+        if link_up:
+            ns_dev.link.set_up()
         if self.conf.ovs_use_veth:
             root_dev.link.set_up()
 
@@ -442,7 +473,8 @@ class BridgeInterfaceDriver(LinuxInterfaceDriver):
     DEV_NAME_PREFIX = 'ns-'
 
     def plug_new(self, network_id, port_id, device_name, mac_address,
-                 bridge=None, namespace=None, prefix=None, mtu=None):
+                 bridge=None, namespace=None, prefix=None, mtu=None,
+                 link_up=True):
         """Plugin the interface."""
         ip = ip_lib.IPWrapper()
 
@@ -461,7 +493,8 @@ class BridgeInterfaceDriver(LinuxInterfaceDriver):
             LOG.warning("No MTU configured for port %s", port_id)
 
         root_veth.link.set_up()
-        ns_veth.link.set_up()
+        if link_up:
+            ns_veth.link.set_up()
 
     def unplug(self, device_name, bridge=None, namespace=None, prefix=None):
         """Unplug the interface."""
