@@ -499,6 +499,10 @@ class IpLinkCommand(IpDeviceCommandBase):
         return privileged.get_link_attributes(self.name,
                                               self._parent.namespace)
 
+    @property
+    def exists(self):
+        return privileged.interface_exists(self.name, self._parent.namespace)
+
 
 class IpAddrCommand(IpDeviceCommandBase):
     COMMAND = 'addr'
@@ -1013,12 +1017,21 @@ def network_namespace_exists(namespace, try_is_ready=False, **kwargs):
     return False
 
 
+def list_namespace_pids(namespace):
+    """List namespace process PIDs
+
+    :param namespace: (string) the name of the namespace
+    :return: (tuple)
+    """
+    return privileged.list_ns_pids(namespace)
+
+
 def ensure_device_is_ready(device_name, namespace=None):
     dev = IPDevice(device_name, namespace=namespace)
     try:
         # Ensure the device has a MAC address and is up, even if it is already
-        # up. If the device doesn't exist, a RuntimeError will be raised.
-        if not dev.link.address:
+        # up.
+        if not dev.link.exists or not dev.link.address:
             LOG.error("Device %s cannot be used as it has no MAC "
                       "address", device_name)
             return False
