@@ -23,6 +23,7 @@ from neutron_lib.exceptions import l3 as l3_exc
 from neutron_lib.plugins import constants as plugin_constants
 from neutron_lib.plugins import directory
 from oslo_config import cfg
+from oslo_log import helpers as log_helpers
 from oslo_log import log as logging
 from sqlalchemy import or_
 
@@ -167,12 +168,18 @@ class L3_DVRsch_db_mixin(l3agent_sch_db.L3AgentSchedulerDbMixin):
             router_ids.add(subnet_port['device_id'])
         return router_ids
 
-    def get_subnet_ids_on_router(self, context, router_id):
+    def get_subnet_ids_on_router(self, context, router_id,
+                                 keep_gateway_port=True):
         """Return subnet IDs for interfaces attached to the given router."""
         subnet_ids = set()
         filter_rtr = {'device_id': [router_id]}
         int_ports = self._core_plugin.get_ports(context, filters=filter_rtr)
+
         for int_port in int_ports:
+            if (not keep_gateway_port and
+                    int_port['device_owner'] ==
+                    n_const.DEVICE_OWNER_ROUTER_GW):
+                continue
             int_ips = int_port['fixed_ips']
             if int_ips:
                 int_subnet = int_ips[0]['subnet_id']
@@ -404,6 +411,9 @@ class L3_DVRsch_db_mixin(l3agent_sch_db.L3AgentSchedulerDbMixin):
             context, agent_db, router_ids, with_dvr))
         if not with_dvr:
             return result_set
+        LOG.debug("Routers %(router_ids)s bound to L3 agent in host %(host)s",
+                  {'router_ids': result_set,
+                   'host': agent_db['host']})
         router_ids = set(router_ids or [])
         if router_ids and result_set == router_ids:
             # no need for extra dvr checks if requested routers are
@@ -423,20 +433,28 @@ class L3_DVRsch_db_mixin(l3agent_sch_db.L3AgentSchedulerDbMixin):
             else:
                 for router_id in (router_ids - result_set):
                     subnet_ids = self.get_subnet_ids_on_router(
-                        context, router_id)
+                        context, router_id, keep_gateway_port=False)
                     if (subnet_ids and
                             self._check_dvr_serviceable_ports_on_host(
                                     context, agent_db['host'],
                                     list(subnet_ids))):
                         result_set.add(router_id)
 
-            for dvr_router in dvr_routers:
+            LOG.debug("Routers %(router_ids)s are scheduled or have "
+                      "serviceable ports in host %(host)s",
+                      {'router_ids': result_set,
+                       'host': agent_db['host']})
+            for router_id in router_ids:
                 result_set |= set(
                     self._get_other_dvr_router_ids_connected_router(
-                        context, dvr_router))
+                        context, router_id))
 
+        LOG.debug("Router IDs %(router_ids)s for agent in host %(host)s",
+                  {'router_ids': result_set,
+                   'host': agent_db['host']})
         return list(result_set)
 
+    @log_helpers.log_method_call
     def _check_dvr_serviceable_ports_on_host(self, context, host, subnet_ids):
         """Check for existence of dvr serviceable ports on host
 
