@@ -33,6 +33,7 @@ import six
 from sqlalchemy.orm import scoped_session
 
 from neutron._i18n import _
+from neutron.common import _constants as const
 from neutron.common import constants as n_const
 from neutron.common import utils
 from neutron.db.models import securitygroup as sg_models
@@ -41,6 +42,7 @@ from neutron.extensions import securitygroup as ext_sg
 from neutron.objects import base as base_obj
 from neutron.objects import ports as port_obj
 from neutron.objects import securitygroup as sg_obj
+from neutron import quota
 
 
 @resource_extend.has_resource_extenders
@@ -108,6 +110,12 @@ class SecurityGroupDbMixin(ext_sg.SecurityGroupPluginBase,
                 name=s['name'], is_default=default_sg)
             sg.create()
 
+            delta = len(ext_sg.sg_supported_ethertypes)
+            delta = delta * 2 if default_sg else delta
+            reservation = quota.QUOTAS.make_reservation(
+                context, tenant_id, {'security_group_rule': delta},
+                self)
+
             for ethertype in ext_sg.sg_supported_ethertypes:
                 if default_sg:
                     # Allow intercommunication
@@ -127,6 +135,9 @@ class SecurityGroupDbMixin(ext_sg.SecurityGroupPluginBase,
                 sg.rules.append(egress_rule)
             sg.obj_reset_changes(['rules'])
 
+            quota.QUOTAS.commit_reservation(context,
+                                            reservation.reservation_id)
+
             # fetch sg from db to load the sg rules with sg model.
             sg = sg_obj.SecurityGroup.get_object(context, id=sg.id)
             secgroup_dict = self._make_security_group_dict(sg)
@@ -476,8 +487,8 @@ class SecurityGroupDbMixin(ext_sg.SecurityGroupPluginBase,
         ip_proto = self._get_ip_proto_number(rule['protocol'])
         # Not all firewall_driver support all these protocols,
         # but being strict here doesn't hurt.
-        if (ip_proto in n_const.SG_PORT_PROTO_NUMS or
-                ip_proto in n_const.SG_PORT_PROTO_NAMES):
+        if (ip_proto in const.SG_PORT_PROTO_NUMS or
+                ip_proto in const.SG_PORT_PROTO_NAMES):
             if rule['port_range_min'] == 0 or rule['port_range_max'] == 0:
                 raise ext_sg.SecurityGroupInvalidPortValue(port=0)
             elif (rule['port_range_min'] is not None and
@@ -503,7 +514,7 @@ class SecurityGroupDbMixin(ext_sg.SecurityGroupPluginBase,
             if (rule['port_range_min'] is not None or
                     rule['port_range_max'] is not None):
                 port_protocols = (
-                    ', '.join(s.upper() for s in n_const.SG_PORT_PROTO_NAMES))
+                    ', '.join(s.upper() for s in const.SG_PORT_PROTO_NAMES))
                 raise ext_sg.SecurityGroupInvalidProtocolForPort(
                     protocol=ip_proto, valid_port_protocols=port_protocols)
 
@@ -680,6 +691,19 @@ class SecurityGroupDbMixin(ext_sg.SecurityGroupPluginBase,
         pager = base_obj.Pager(
             sorts=sorts, marker=marker, limit=limit, page_reverse=page_reverse)
 
+        project_id = filters.get('project_id') or filters.get('tenant_id')
+        if project_id:
+            project_id = project_id[0]
+        else:
+            project_id = context.project_id
+        if project_id:
+            self._ensure_default_security_group(context, project_id)
+
+        if not filters and context.project_id and not context.is_admin:
+            rule_ids = sg_obj.SecurityGroupRule.get_security_group_rule_ids(
+                context.project_id)
+            filters = {'id': rule_ids}
+
         # NOTE(slaweq): use admin context here to be able to get all rules
         # which fits filters' criteria. Later in policy engine rules will be
         # filtered and only those which are allowed according to policy will
