@@ -17,6 +17,8 @@ from neutron_lib.callbacks import events as callbacks_events
 from neutron_lib.callbacks import registry as callbacks_registry
 from neutron_lib.callbacks import resources as callbacks_resources
 from neutron_lib import constants
+from neutron_lib.utils import helpers
+from oslo_config import cfg
 import testtools
 
 from neutron.agent.common import ovs_lib
@@ -32,12 +34,16 @@ from neutron.plugins.ml2.drivers.openvswitch.agent.openflow.ovs_ofctl \
 from neutron.tests import base
 
 TESTING_VLAN_TAG = 1
+TESTING_SEGMENT = 1000
 
 
-def create_ofport(port_dict):
+def create_ofport(port_dict, network_type=None, physical_network=None):
     ovs_port = mock.Mock(vif_mac='00:00:00:00:00:00', ofport=1,
                          port_name="port-name")
-    return ovsfw.OFPort(port_dict, ovs_port, vlan_tag=TESTING_VLAN_TAG)
+    return ovsfw.OFPort(port_dict, ovs_port, vlan_tag=TESTING_VLAN_TAG,
+                        segment_id=TESTING_SEGMENT,
+                        network_type=network_type,
+                        physical_network=physical_network)
 
 
 class TestCreateRegNumbers(base.BaseTestCase):
@@ -302,9 +308,9 @@ class TestConjIPFlowManager(base.BaseTestCase):
         self.vlan_tag = 100
         self.conj_id = 16
 
-    def test_update_flows_for_vlan_no_ports(self):
+    def test_update_flows_for_vlan_no_members(self):
         remote_group = self.driver.sg_port_map.get_sg.return_value
-        remote_group.ports = {}
+        remote_group.members = {}
         with mock.patch.object(self.manager.conj_id_map,
                                'get_conj_id') as get_conj_id_mock:
             get_conj_id_mock.return_value = self.conj_id
@@ -314,6 +320,21 @@ class TestConjIPFlowManager(base.BaseTestCase):
         self.assertFalse(remote_group.get_ethertype_filtered_addresses.called)
         self.assertFalse(self.driver._add_flow.called)
 
+    def test_update_flows_for_vlan_no_ports_but_members(self):
+        remote_group = self.driver.sg_port_map.get_sg.return_value
+        remote_group.ports = set()
+        remote_group.members = {constants.IPv4: ['10.22.3.4']}
+        remote_group.get_ethertype_filtered_addresses.return_value = [
+            '10.22.3.4']
+        with mock.patch.object(self.manager.conj_id_map,
+                               'get_conj_id') as get_conj_id_mock:
+            get_conj_id_mock.return_value = self.conj_id
+            self.manager.add(self.vlan_tag, 'sg', 'remote_id',
+                             constants.INGRESS_DIRECTION, constants.IPv4, 0)
+            self.manager.update_flows_for_vlan(self.vlan_tag)
+        self.assertTrue(remote_group.get_ethertype_filtered_addresses.called)
+        self.assertTrue(self.driver._add_flow.called)
+
     def test_update_flows_for_vlan(self):
         remote_group = self.driver.sg_port_map.get_sg.return_value
         remote_group.get_ethertype_filtered_addresses.return_value = [
@@ -378,6 +399,7 @@ class TestOVSFirewallDriver(base.BaseTestCase):
         self.fake_ovs_port = FakeOVSPort('port', 1, '00:00:00:00:00:00')
         self.mock_bridge.br.get_vif_port_by_id.return_value = \
             self.fake_ovs_port
+        cfg.CONF.set_override('explicitly_egress_direct', True, 'AGENT')
 
     def _prepare_security_group(self):
         security_group_rules = [
@@ -565,6 +587,135 @@ class TestOVSFirewallDriver(base.BaseTestCase):
             self.firewall.prepare_port_filter(port_dict)
         self.assertFalse(m_init_flows.called)
 
+    def test_initialize_port_flows_vlan_dvr_conntrack_direct(self):
+        port_dict = {
+            'device': 'port-id',
+            'security_groups': [1]}
+        of_port = create_ofport(port_dict,
+                                network_type=constants.TYPE_VXLAN)
+        self.firewall.sg_port_map.ports[of_port.id] = of_port
+        port = self.firewall.get_or_create_ofport(port_dict)
+
+        fake_patch_port = 999
+        self.mock_bridge.br.get_port_ofport.return_value = fake_patch_port
+
+        self.firewall.initialize_port_flows(port)
+
+        call_args1 = {
+            'table': ovs_consts.TRANSIENT_TABLE,
+            'priority': 100,
+            'in_port': port.ofport,
+            'actions': 'set_field:{:d}->reg{:d},'
+                       'set_field:{:d}->reg{:d},'
+                       'resubmit(,{:d})'.format(
+                           port.ofport,
+                           ovsfw_consts.REG_PORT,
+                           port.vlan_tag,
+                           ovsfw_consts.REG_NET,
+                           ovs_consts.BASE_EGRESS_TABLE)}
+        egress_flow_call = mock.call(**call_args1)
+
+        call_args2 = {
+            'table': ovs_consts.TRANSIENT_TABLE,
+            'priority': 90,
+            'dl_dst': port.mac,
+            'dl_vlan': '0x%x' % port.segment_id,
+            'actions': 'set_field:{:d}->reg{:d},'
+                       'set_field:{:d}->reg{:d},'
+                       'strip_vlan,resubmit(,{:d})'.format(
+                           port.ofport,
+                           ovsfw_consts.REG_PORT,
+                           port.vlan_tag,
+                           ovsfw_consts.REG_NET,
+                           ovs_consts.BASE_INGRESS_TABLE)}
+        ingress_flow_call1 = mock.call(**call_args2)
+
+        call_args3 = {
+            'table': ovs_consts.TRANSIENT_TABLE,
+            'priority': 90,
+            'dl_dst': port.mac,
+            'dl_vlan': '0x%x' % port.vlan_tag,
+            'actions': 'set_field:{:d}->reg{:d},'
+                       'set_field:{:d}->reg{:d},'
+                       'strip_vlan,resubmit(,{:d})'.format(
+                           port.ofport,
+                           ovsfw_consts.REG_PORT,
+                           port.vlan_tag,
+                           ovsfw_consts.REG_NET,
+                           ovs_consts.BASE_INGRESS_TABLE)}
+        ingress_flow_call2 = mock.call(**call_args3)
+        self.mock_bridge.br.add_flow.assert_has_calls(
+            [egress_flow_call, ingress_flow_call1, ingress_flow_call2])
+
+    def test_initialize_port_flows_vlan_dvr_conntrack_direct_vlan(self):
+        port_dict = {
+            'device': 'port-id',
+            'security_groups': [1]}
+        of_port = create_ofport(port_dict,
+                                network_type=constants.TYPE_VLAN,
+                                physical_network='vlan1')
+        self.firewall.sg_port_map.ports[of_port.id] = of_port
+        port = self.firewall.get_or_create_ofport(port_dict)
+
+        fake_patch_port = 999
+        self.mock_bridge.br.get_port_ofport.return_value = fake_patch_port
+
+        with mock.patch.object(helpers, "parse_mappings",
+                               return_value={"vlan1": "br-vlan1"}):
+            self.firewall.initialize_port_flows(port)
+
+    def test_delete_all_port_flows(self):
+        port_dict = {
+            'device': 'port-id',
+            'security_groups': [1]}
+        of_port = create_ofport(port_dict,
+                                network_type=constants.TYPE_VXLAN)
+        self.firewall.sg_port_map.ports[of_port.id] = of_port
+        port = self.firewall.get_or_create_ofport(port_dict)
+
+        self.firewall.delete_all_port_flows(port)
+
+        call_args1 = {"strict": True,
+                      "priority": 90,
+                      "table": ovs_consts.TRANSIENT_TABLE,
+                      "dl_dst": port.mac,
+                      "dl_vlan": port.vlan_tag}
+        flow1 = mock.call(**call_args1)
+
+        call_args2 = {"strict": True,
+                      "priority": 90,
+                      "table": ovs_consts.TRANSIENT_TABLE,
+                      "dl_dst": port.mac,
+                      "dl_vlan": port.segment_id}
+        flow2 = mock.call(**call_args2)
+
+        call_args3 = {"table": ovs_consts.ACCEPT_OR_INGRESS_TABLE,
+                      "dl_dst": port.mac,
+                      "reg6": port.vlan_tag}
+        flow3 = mock.call(**call_args3)
+
+        call_args4 = {"in_port": port.ofport,
+                      "strict": True,
+                      "priority": 100,
+                      "table": ovs_consts.TRANSIENT_TABLE}
+        flow4 = mock.call(**call_args4)
+
+        call_args5 = {"reg5": port.ofport}
+        flow5 = mock.call(**call_args5)
+
+        call_args6 = {"table": ovs_consts.ACCEPTED_EGRESS_TRAFFIC_NORMAL_TABLE,
+                      "dl_dst": port.mac,
+                      "reg6": port.vlan_tag}
+        flow6 = mock.call(**call_args6)
+
+        call_args7 = {"table": ovs_consts.ACCEPTED_EGRESS_TRAFFIC_NORMAL_TABLE,
+                      "dl_src": port.mac,
+                      "reg6": port.vlan_tag}
+        flow7 = mock.call(**call_args7)
+
+        self.mock_bridge.br.delete_flows.assert_has_calls(
+            [flow1, flow2, flow3, flow6, flow7, flow4, flow5])
+
     def test_prepare_port_filter_initialized_port(self):
         port_dict = {'device': 'port-id',
                      'security_groups': [1]}
@@ -730,7 +881,8 @@ class TestOVSFirewallDriver(base.BaseTestCase):
 
     def test__remove_egress_no_port_security_deletes_flow(self):
         self.mock_bridge.br.db_get_val.return_value = {'tag': TESTING_VLAN_TAG}
-        self.firewall.sg_port_map.unfiltered['port_id'] = 1
+        self.firewall.sg_port_map.unfiltered['port_id'] = (
+            self.fake_ovs_port, 100)
         self.firewall._remove_egress_no_port_security('port_id')
         expected_call = mock.call(
             table=ovs_consts.TRANSIENT_TABLE,
@@ -748,9 +900,10 @@ class TestOVSFirewallDriver(base.BaseTestCase):
         with mock.patch.object(self.firewall.int_br.br, 'get_vifs_by_ids',
                                return_value={'port_id': vif_port}):
             self.firewall.process_trusted_ports(['port_id'])
-            self.assertEqual(1, len(self.firewall.sg_port_map.unfiltered))
-            self.assertEqual(vif_port.ofport,
-                             self.firewall.sg_port_map.unfiltered['port_id'])
+            self.assertEqual(1,
+                             len(self.firewall.sg_port_map.unfiltered.keys()))
+            ofport, _ = self.firewall.sg_port_map.unfiltered['port_id']
+            self.assertEqual(vif_port.ofport, ofport.ofport)
 
     def test_process_trusted_ports_port_not_found(self):
         """Check that exception is not propagated outside."""
@@ -761,7 +914,8 @@ class TestOVSFirewallDriver(base.BaseTestCase):
             self.assertEqual(0, len(self.firewall.sg_port_map.unfiltered))
 
     def test_remove_trusted_ports_clears_cached_port_id(self):
-        self.firewall.sg_port_map.unfiltered['port_id'] = 1
+        self.firewall.sg_port_map.unfiltered['port_id'] = (
+            self.fake_ovs_port, 100)
         self.firewall.remove_trusted_ports(['port_id'])
         self.assertNotIn('port_id', self.firewall.sg_port_map.unfiltered)
 
